"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = require("path");
const readProjectConfig_1 = require("../tools/readProjectConfig");
const log_1 = require("../common/log");
const const_1 = require("../common/const");
const util_1 = require("../common/util");
const child_process_1 = require("child_process");
const fs_1 = require("fs");
exports.command = "test";
exports.desc = "单元测试";
exports.builder = (yargs) => {
    return yargs
        .option("p", {
        alias: "path",
        describe: "测试的目录",
        default: ".",
        type: "string"
    })
        .usage("Usage: $0 test -p .");
};
exports.handler = (yargs) => __awaiter(this, void 0, void 0, function* () {
    let projectPath = path_1.resolve(yargs.p) || process.cwd();
    let configFileName = path_1.join(projectPath, const_1.projectConfigFile);
    let projectConfig = yield readProjectConfig_1.readProjectConfig(configFileName);
    if (projectConfig.unitTest) {
        let mochapars = getMochapars(yargs.mochapars);
        let nycrcFile = createNycrcFile(projectPath);
        test(projectPath, mochapars);
        coveralls(projectPath, mochapars);
        fs_1.unlinkSync(nycrcFile);
    }
});
function getMochapars(mochapars) {
    let pars = [];
    if (mochapars) {
        console.log(`mocha参数：${mochapars}`);
        mochapars.split(",").map(p => {
            let kv = p.split("=");
            pars.push(`--${kv[0]}`);
            pars.push(`${kv[1]}`);
        });
    }
    return pars;
}
function createNycrcFile(projectPath) {
    let nycrcFile = path_1.join(projectPath, ".nycrc");
    fs_1.writeFileSync(nycrcFile, `{
  "include": [
    "src/**/*.ts"
  ],
  "extension": [
    ".ts"
  ],
  "require": [
    "ts-node/register"
  ],
  "sourceMap": true,
  "instrument": true
}`.trim());
    return nycrcFile;
}
function test(projectPath, mochapars) {
    log_1.printMessage("单元测试开始...");
    let nyc = util_1.getCreateProjectDependencies(projectPath, path_1.join("nyc", "bin", "nyc.js"));
    let mocha = util_1.getCreateProjectDependencies(projectPath, path_1.join("mocha", "bin", "mocha"));
    let options = {
        encoding: "utf8",
        cwd: projectPath,
        stdio: [process.stdin, process.stdout, process.stderr]
    };
    log_1.printMessage(`执行命令：${nyc} ${mocha} ${mochapars.join(" ")}`);
    let childProcess = child_process_1.spawnSync(nyc, [mocha].concat(mochapars), options);
    if (childProcess.status !== 0) {
        if (childProcess.error && childProcess.error.message) {
            log_1.printErrorMessage(childProcess.error.message);
        }
        if (childProcess.stderr) {
            log_1.printErrorMessage(childProcess.stderr);
        }
        log_1.printErrorMessage("单元测试不通过！");
        process.exit(childProcess.status);
        return;
    }
    log_1.printMessage("单元测试成功结束");
}
function coveralls(projectPath, mochapars) {
    log_1.printMessage("覆盖率开始...");
    let nyc = util_1.getCreateProjectDependencies(projectPath, path_1.join("nyc", "bin", "nyc.js"));
    let coveralls = util_1.getCreateProjectDependencies(projectPath, path_1.join("coveralls", "bin", "coveralls.js"));
    let commandText = `"${nyc}" mocha ${mochapars.join(" ")} && "${nyc}" report --reporter=text-lcov | "${coveralls}"`;
    log_1.printMessage(`执行命令：${commandText}`);
    let res = child_process_1.execSync(commandText, { encoding: "utf-8", cwd: projectPath });
    log_1.printMessage("覆盖率完成");
}
//# sourceMappingURL=test.js.map